#!/bin/bash
LOG_FILE=/opt/configuracio/Control_APT.log
if [ ! -d /opt/configuracio ]; then
{
  LOG_FILE=/var/log/Control_APT.log
}
fi

esRoot() {

   if [ $(/usr/bin/id -u) -ne 0 ]; then
    {
      echo "No ets root, es surt del programa" 2> /dev/null
      exit 1
    }
    fi
}

iniciLog() {

  echo "$(date +%F--%T) - #### INICIA $1 ####" >> $LOG_FILE

}


finalLog() {

  echo "$(date +%F--%T) - #### FINALITZA $1 ####" >> $LOG_FILE
  echo "$(date +%F--%T) -" >> $LOG_FILE

}

comprovaErrades() {
  if [ $? -ne 0 ]; then
      echo "Si que ha hagut errades mira $LOG_FILE per detalls."
  else
      echo "Funció aplicada amb èxit, pots revisar $LOG_FILE."
  fi
}

aturaRepo() {
    echo "Mato tot el procés APT"
    ps aux | grep apt | awk -F ' ' '{print $2}' | xargs -I {} kill -9 {} 2>/dev/null

    echo "--------------------------------------"
    echo "Desbloco els repositoris"
    echo "--------------------------------------"

    if [ -f /var/lib/apt/lists/lock ]
    then
        {
            echo "Desbloquem repositoris"
            rm /var/lib/apt/lists/lock
        } else {
            echo "Podem continuar"
            
        } 
    fi

    if [ -f /var/lib/dpkg/lock ]
    then
        {
            echo "Desbloquem paquets"
            rm /var/lib/dpkg/lock
        } else {
            echo "Podem continuar"
        } 
    fi

    if [ -f /var/lib/dpkg/lock-frontend ]
    then
        {
            echo "Desbloquem el forntend de paquetari"
            rm /var/lib/dpkg/lock-frontend
        } else  {
            echo "Podem continuar"
        }
    fi

    echo "--------------------------------------"
    echo " Repara paquets mal instal·lats"
    echo "--------------------------------------"

    dpkg --configure -a

}

actualitza() {
  echo "--------------------------------------"
  echo " Repara paquets mal instal·lats"
  echo "--------------------------------------"
  dpkg --configure -a

  echo "--------------------------------------"
  echo "Actualització dels repositoris"
  echo "--------------------------------------"

  apt update

  echo ""
  echo "--------------------------------------"
  echo "Reparo els possibles paquets trencats "
  echo "--------------------------------------"
  echo ""
  apt -f -y install 

  echo ""
  echo "--------------------------------------"
  echo "Llista dels paquets actualitzables"
  echo "--------------------------------------"
  echo ""
  apt list --upgradable

  echo ""
  echo "--------------------------------------"
  echo "Actualitzant els paquets amb la configuració per defecte"
  echo "--------------------------------------"
  echo ""

  apt-get -y -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" upgrade

  echo ""
  echo "--------------------------------------"
  echo "Actualització dels snaps"
  echo "--------------------------------------"
  echo ""

  snap refresh

  if [ -f /usr/bin/flatpak ] ; then
  {
    echo ""
    echo "--------------------------------------"
    echo "Actualització dels flatpak"
    echo "--------------------------------------"
    echo ""

    flatpak update
    comprovaErrades
  }
  fi
  

}

cridaCA(){
  echo ""
  echo "--------------------------------------"
  echo "Equip actualitzat, Comunico amb CA"
  echo "--------------------------------------"
  echo ""

  . /etc/profile.CA all 
  caf start sdagent amagent
  
}

##Comença la instalació 
esRoot 

iniciLog "---- SCRIPT ----"

##Executo la funció
aturaRepo >> $LOG_FILE 2> /dev/null
comprovaErrades >> $LOG_FILE

actualitza >> $LOG_FILE 2> /dev/null
comprovaErrades >> $LOG_FILE

cridaCA >> $LOG_FILE 2> /dev/null
comprovaErrades >> $LOG_FILE

finalLog "---- SCRIPT ----"


exit $?

