#!/bin/bash

LOG_FILE=/opt/configuracio/Control_APT.log
FLATPAK_BIN=$(which flatpak)
SNAP_BIN=$(which snap)
DIALOG_BIN=$(which dialog)
CONKY_CONF=/etc/conky/conky.conf
CONKY_CONF_ORIG=/etc/conky/conky.conf.orig

if [ ! /opt/configuracio]; then
  mkdir -p /opt/configuracio
fi

esRoot() {
  if [ $(/usr/bin/id -u) -ne 0 ]; then
    echo "No ets root, es surt del programa" >&2
    exit 1
  fi
}

iniciLog() {
  echo "$(date +%F--%T) - #### INICIA $1 ####" >>"$LOG_FILE"
}

finalLog() {
  echo "$(date +%F--%T) - #### FINALITZA $1 ####" >>"$LOG_FILE"
  echo "$(date +%F--%T) -" >>"$LOG_FILE"
}

comprovaErrades() {
  if [ $? -ne 0 ]; then
    echo "Si que ha hagut errades mira $LOG_FILE per detalls." >>"$LOG_FILE"
  else
    echo "Funció aplicada amb èxit, pots revisar $LOG_FILE." >>"$LOG_FILE"
  fi
}

NotificaUsuaris() {
  $DIALOG_BIN --infobox "Comença el procediment per la reparació i actualització!!" 5 40
  sleep 4
  echo "Es notifica per conky $CONKY_CONF"
  if [ ! -f $CONKY_CONF ]; then
    {
      echo "ATENCIO!! No existeix el fitxer $CONKY_CONF !!"
    }
  else
    {
      echo "Faig una còpia de seguretat del fitxer $CONKY_CONF a $CONKY_CONF_ORIG"
      cp -v $CONKY_CONF $CONKY_CONF_ORIG
      echo "Modifico el fitxer $CONKY_CONF per mostrar el missatge"
      sudo sed -i '/]]/i ${alignr}${color black}EQUIP ACTUALITZANT-SE,\nEL TEXT DESAPAREIXERÀ EN FINALITZAR' $CONKY_CONF
    }
  fi
}

NotificaUsuarisFinal() {
  echo "Tasca finalitzada"
  echo "Es notifica per conky per recuperar el fitxer original"
  if [ ! -f $CONKY_CONF_ORIG ]; then
    {
      echo "ATENCIO!! No existeix el fitxer $CONKY_CONF_ORIG !!"
    }
  else
    {
      echo "Recupero la còpia original de $CONKY_CONF"
      cp -v $CONKY_CONF_ORIG $CONKY_CONF
    }
  fi
  $DIALOG_BIN --infobox "Equip reparat i actualitzat, es recomana reinici" 5 40
  sleep 5
}

ReparaAinici() {
  echo "Es recuperarà a l'inici"

  if [ -f /opt/configuracio/Control_APT_KO ]; then
    rm /etc/cron.d/linkat-actualitza-cron
    rm /opt/configuracio/lkupdate
    rm /opt/configuracio/Control_APT_KO
  else
    echo "SHELL=/bin/bash" >/etc/cron.d/linkat-actualitza-cron
    echo "@reboot root /opt/configuracio/lkupdate" >>/etc/cron.d/linkat-actualitza-cron

    cat <<EOF | tee /opt/configuracio/lkupdate
#!/bin/bash
CONKY_CONF=/etc/conky/conky.conf
CONKY_CONF_ORIG=/etc/conky/conky.conf.orig
if [ -f /opt/configuracio/Control_APT_KO ]; then
    rm /etc/cron.d/linkat-actualitza-cron
    rm /opt/configuracio/lkupdate
    rm /opt/configuracio/Control_APT_KO
    exit 
fi
ps aux | grep apt | awk -F ' ' '{print \$2}' | xargs -I {} kill -9 {} 2>/dev/null
rm /var/lib/apt/lists/lock && rm /var/lib/dpkg/lock && rm /var/lib/dpkg/lock-frontend && DEBIAN_FRONTEND=noninteractive dpkg --configure -a
DEBIAN_FRONTEND=noninteractive apt-get -y -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" full-upgrade
snap refresh
flatpak update -y

echo "Creo un fitxer per dir que tot ha anat be"
touch /opt/configuracio/Control_APT_KO
echo "Recupero la còpia original de $CONKY_CONF"
cp -v $CONKY_CONF_ORIG $CONKY_CONF
EOF
    chmod a+x /opt/configuracio/lkupdate
    echo "funció de Repara Inici aplicada"
  fi
}

aturaRepo() {
  echo "Mato tot el procés APT"
  $DIALOG_BIN --infobox "Mato els procés APT ..." 5 40
  sleep 4
  ps aux | grep apt | awk -F ' ' '{print $2}' | xargs -I {} kill -9 {} 2>/dev/null
  echo "Desbloco els repositoris"
  $DIALOG_BIN --infobox "Desbloco repositoris ..." 5 40
  sleep 4
  rm /var/lib/apt/lists/lock 2>/dev/null
  rm /var/lib/dpkg/lock 2>/dev/null
  rm /var/lib/dpkg/lock-frontend 2>/dev/null
  DEBIAN_FRONTEND=noninteractive dpkg --configure -a
}

actualitza() {
  $DIALOG_BIN --infobox "Configurem els paquets pendents ..." 5 40
  sleep 4
  DEBIAN_FRONTEND=noninteractive dpkg --configure -a
  $DIALOG_BIN --infobox "Actualitzem els paquets pendents ..." 5 40
  sleep 4
  apt update
  $DIALOG_BIN --infobox "Instal·lem les dependències que falten ..." 5 40
  sleep 4
  DEBIAN_FRONTEND=noninteractive apt -f -y install
  $DIALOG_BIN --infobox "Actualitzem els paquets de la Linkat ..." 5 40
  sleep 4
  DEBIAN_FRONTEND=noninteractive apt-get -y -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold" full-upgrade
  #
  # REFRESQUEM PAQUETS SNAP
  #
  systemctl restart snapd
  $DIALOG_BIN --infobox "Actualitzem els paquets SNAP ..." 5 40
  sleep 4
  $SNAP_BIN refresh
  #
  # ELIMINEM PAQUETS SNAP ANTICS
  #
  $DIALOG_BIN --infobox "Eliminem els paquets antics SNAP ..." 5 40
  sleep 4
  set -eu
  LANG=en_US.UTF-8 snap list --all | awk '/disabled/{print $1, $3}' |
    while read snapname revision; do
      $SNAP_BIN remove "$snapname" --revision="$revision"
    done
  #
  # REFRESQUEM PAQUETS FLATPAK
  #
  $DIALOG_BIN --infobox "Actualitzem els paquets FLATPAK ..." 5 40

  sleep 4
  $FLATPAK_BIN update -y
  $DIALOG_BIN --infobox "Actualització del sistema finalitzada ..." 5 40
  sleep 4
}

# Comença la instal·lació
esRoot
#iniciLog "---- SCRIPT ----"
NotificaUsuaris "Actualització en curs..." 2>/dev/null
ReparaAinici >>"$LOG_FILE" 2>/dev/null
aturaRepo 2>/dev/null
comprovaErrades >>"$LOG_FILE" 2>/dev/null
actualitza 2>/dev/null
comprovaErrades >>"$LOG_FILE" 2>/dev/null
ReparaAinici
NotificaUsuarisFinal "Actualització finalitzada" 2>/dev/null
#finalLog "---- FINTAL SCRIPT ----"

exit 0
